module specify_file_names

 ! Using Compaq Visual Fortran routines, functions determined below open the 
 ! Windows APIs to open dialogs to specify file names, i.e., data file and the 
 ! file to which results are stored. The files is expected to have the file 
 ! extension "fdt" and "mim" respectively. 
 !
 ! This is not compatible for compilers other than the Visual Fortran. If you 
 ! use one of the compilers, do not use this module. 
 !
 ! Version    : 2.2 (February,  2005) M. Otsubo and A. Yamaji
 !              rename function specify_cgp_file_name -> get_path_filename_extension
 ! Version    : 2.1 (September, 2004) K. Sato
 !              add function specify_cgp_file_name
 ! Version    : 2 (January, 2001) A. Yamaji
 ! Version    : 1                 A. Yamaji
 !

!/////////////////////////// Decraration Section ///////////////////////////////

  use DFWIN ! Visual Fortran routines 
  implicit none; private

! public subprograms ==========

  public specify_open_file_name
  public specify_save_file_name
  public get_path_filename_extension
  public get_home_directory

! private =====================

  type(T_OPENFILENAME), private :: ofn
  character(len=512),   private :: file_spec = ""C
  integer,              private :: status, ilen

contains !///////////////////// Subprograms ////////////////////////////////////

subroutine determine_TOPENFILENAME() 
  ofn%lStructSize = SIZEOF(ofn)
  ofn%hwndOwner         = NULL
  ofn%hInstance         = NULL
  ofn%lpstrCustomFilter = NULL
  ofn%nMaxCustFilter    = 0
  ofn%nFilterIndex      = 1
  ofn%lpstrFile         = loc(file_spec)
  ofn%nMaxFile          = sizeof(file_spec)
  ofn%nMaxFileTitle     = 0
  ofn%lpstrInitialDir   = NULL
  ofn%lpstrTitle        = loc(""C)
  ofn%Flags             = OFN_PATHMUSTEXIST
  ofn%lpstrDefExt       = loc("txt"C)
  ofn%lpfnHook          = NULL
  ofn%lpTemplateName    = NULL
end subroutine determine_TOPENFILENAME

!-------------------------------------------------------------------------------
function specify_open_file_name() result (filename)
  character (300) :: filename

  character*(*),parameter :: filter_spec =      &
      "Geological Fault-Slip Data (*.fdt)"C   // "*.fdt"C //  &
      "Seismic Focal Mechanism Data (*.fmd)"C // "*.fmd"C //  &
      "All Files (*.*)"C                      // "*.*"C   //""C

  call determine_TOPENFILENAME()
  ofn%lpstrFilter = loc(filter_spec)

  status = GetOpenFileName(ofn)
  if (status .eq. 0) then
    write (*,*) "No data file specified --- Execution terminated."
	stop
  else
    ilen = INDEX(file_spec,CHAR(0))
    filename = file_spec(1:ilen-1)
  end if
end function specify_open_file_name

!-------------------------------------------------------------------------------
function specify_save_file_name( filter_spec ) result (filename)
  character*(*), intent(in) :: filter_spec
  character (len=300) :: filename

  call determine_TOPENFILENAME()
  ofn%lpstrFilter = loc(filter_spec)

  status = GetSaveFileName(ofn)
  if (status .eq. 0) then
    write (*,*) "No data file specified --- Execution terminated."
	stop
  else
    ilen = INDEX(file_spec,CHAR(0))
    filename = file_spec(1:ilen-1)
  end if
end function specify_save_file_name

!-------------------------------------------------------------------------------
subroutine get_path_filename_extension( path, name, ext )
  use DFLIB
  character(300), intent(in)            :: path
  character(256), intent(out), optional :: ext
  character(3)     :: drive
  character(256)   :: dir
  character(256)   :: name
  character(256)   :: ext0
  integer(4)       :: length

  length = SPLITPATHQQ(path, drive, dir, name, ext0)
  if (present(ext)) ext = ext0
end subroutine get_path_filename_extension

!-------------------------------------------------------------------------------
subroutine get_home_directory( dr )
  USE DFLIB
  character(len=300), intent(out) :: dr ! directory
  integer (4)         :: er
  CHARACTER($MAXPATH) :: dir

  dir = FILE$CURDRIVE
  er  = GETDRIVEDIRQQ(dir)
  if (er.eq.0) then
    write (*,*) "*** Could'nt get home directory ***"
	write (*,*) "***     Execution terminated    ***"
	stop
  end if
  dr  = dir
end subroutine get_home_directory

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
end module specify_file_names

